import {
  select,
  json,
  scaleLinear,
  extent,
  axisLeft,
  axisBottom,
  format,
  timeFormat,
  scaleOrdinal,
  schemeCategory10
} from 'd3';

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

const render = data => {
  
  const div = d3.select("body").append("div")
  .attr("class", "tooltip")
  .attr("id", "tooltip")
  .style("opacity", 0);
  
  const title = 'Doping in Professional Bicycle Racing';
  
  const xValue = d => d.Year;
  const xAxisLabel = 'Year';
  
  const yValue = d => d.Time;
  const circleRadius = 10;
  const yAxisLabel = 'Time';
  
  const margin = { top: 60, right: 40, bottom: 88, left: 150 };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
  const colorScale = scaleOrdinal(schemeCategory10);
  
  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();
  
  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
    .nice();
  
  const g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);
  
  let timeFormat = d3.timeFormat("%M:%S");
  
  
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(15)
    .tickFormat(d3.format("d"));
  
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10)
    .tickFormat(timeFormat);
  
  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();
  
  yAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', -93)
      .attr('x', -innerHeight / 2)
      .attr('fill', 'black')
      .attr('transform', `rotate(-90)`)
      .attr('text-anchor', 'middle')
      .text(yAxisLabel);
  
  const xAxisG = g.append('g').call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);
  
  xAxisG.select('.domain').remove();
  
  xAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', 75)
      .attr('x', innerWidth / 2)
      .attr('fill', 'black')
      .text(xAxisLabel);
  
  g.selectAll('circle').data(data)
    .enter().append('circle')
      .attr('cy', d => yScale(yValue(d)))
      .attr('cx', d => xScale(xValue(d)))
      .attr('r', circleRadius)
      .style("fill", function(d) {
      return colorScale(d.Doping != "");
    })
     .on("mouseover", function(d) {
      div.style("opacity", .9);
      //div.attr("data-year", d.Year)
      div.html(d.Name + ": " + d.Nationality + "<br/>"
              + "Year: " +  d.Year + ", Time: " + timeFormat(d.Time) 
              + (d.Doping?"<br/><br/>" + d.Doping:""))
        .style("left", (d3.event.pageX) + "px")
        .style("top", (d3.event.pageY - 28) + "px");
    })
    .on("mouseout", function(d) {
      div.style("opacity", 0);
    });
       
  
  g.append('text')
      .attr('class', 'title')
      .attr('y', -20)
      .attr('x',80)
      .text(title);
   
  const legend = g.selectAll(".legend")
                  .data(colorScale.domain())
                  .enter().append("g")
                  .attr("class", "legend")
                  .attr("id", "legend")
                  .attr("transform", function(d, i) {
                  return "translate(0," + (innerHeight/2 - i * 20) + ")";
    });
  
  legend.append("rect")
    .attr("x", innerWidth - 18)
    .attr("y", 30)
    .attr("width", 18)
    .attr("height", 18)
    .style("fill", colorScale)
    .style('opacity', 0.8);
  
  
  legend.append("text")
    .attr("x", innerWidth - 24)
    .attr("y", 38)
    .attr("dy", ".35em")
    .style("text-anchor", "end")
    .text(function(d) {
      if (d) return "Riders with doping allegations";
      else {
        return "No doping allegations";
      };
    });
  
  
};

json('https://raw.githubusercontent.com/FreeCodeCamp/ProjectReferenceData/master/cyclist-data.json')
  .then(data => {
    data.forEach(d => {
    d.Place = +d.Place;
    var parsedTime = d.Time.split(':');
    d.Time = new Date(1970, 0, 1, 0, parsedTime[0], parsedTime[1]);
    });
    render(data);
  });